#/******************************************************************************
# * $Id$
# *
# * Project:  Perform zonal statistics required for landing site analysis
# * Purpose:  Create virtual mosaic from a directory of images, clip to one or
#             more features, perform selected zonal statistics, and collate 
#             outputs.
# * Author:   Marc Hunter, mahunter@usgs.gov
# *
# ******************************************************************************
# * Public domain licenes (unlicense)
# *
# * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# * DEALINGS IN THE SOFTWARE.
# ****************************************************************************/

import arcpy, os, sys, math, csv
import numpy as np
import pandas as pd
from pandas.tools.plotting import table
import random
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
matplotlib.style.use('ggplot')
from arcpy import env
from arcpy.sa import *
arcpy.env.overwriteoutput = 1

class Toolbox(object):
    def __init__(self):
        """Define the toolbox (the name of the toolbox is the name of the
        .pyt file)."""
        self.label = "Zonal Statistics Tool"
        self.alias = ""

# List of tool classes associated with this toolbox
        self.tools = [MergeFiles, TopologyCheck, ZonalStats, HistogramPlot]

class MergeFiles(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Merge Files"
        self.description = ""
        self.canRunInBackground = False

    def getParameterInfo(self):
        """Define parameter definitions"""
        param0 = arcpy.Parameter(
            displayName="Workspace",
            name="workspace",
            datatype=["DEWorkspace", "DEFeatureDataset"],
            parameterType="Required",
            direction="Input")
        param1 = arcpy.Parameter(
            displayName="Wildcard",
            name="wildcard",
            datatype="GPString",
            parameterType="Optional",
            direction="Input")
        param2 = arcpy.Parameter(
            displayName="Feature Type",
            name="featureType",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        param3 = arcpy.Parameter(
            displayName="Recursive",
            name="recursive",
            datatype="GPBoolean",
            parameterType="Optional",
            direction="Input")
        param4 = arcpy.Parameter(
            displayName="Output Feature Class",
            name="outFC",
            datatype="DEFeatureClass",
            parameterType="Required",
            direction="Output")
        parameters = [param0, param1, param2, param3, param4]
        param2.filter.list = ["POLYGON", "POLYLINE", "POINT", "MULTIPOINT"]
        return parameters

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
# Script arguments

        def recursive_list_fcs(workspace, wild_card=None, feature_type=None):
            """Returns a list of all feature classes in a tree.  Returned
            list can be limited by a wildcard, and feature type.
            """
            preexisting_wks = arcpy.env.workspace
            arcpy.env.workspace = workspace
            try:
                list_fcs = []
                for root, dirs, files in os.walk(workspace):
                    arcpy.env.workspace = root
                    fcs = arcpy.ListFeatureClasses(wild_card, feature_type)
                    if fcs:
                        list_fcs += [os.path.join(root, fc) for fc in fcs]
            except Exception as err:
                raise err
            finally:
                arcpy.env.workspace = preexisting_wks
            return list_fcs
        
        Workspace = parameters[0].valueAsText
        Wildcard = parameters[1].valueAsText
        FeatureType = parameters[2].valueAsText
        Recursive = parameters[3].valueAsText
        outputFC = parameters[4].valueAsText
        
        if Recursive == '#' or not Recursive:
            Recursive = "false" # provide a default value if unspecified

# Local variables:
        FieldName = "Filename"
        outputFCpath, outputFCname = os.path.split(outputFC)

# Process: Iterate Feature Classes
        arcpy.env.workspace = Workspace
        if Recursive == "true":
            WorkspaceList = recursive_list_fcs(Workspace, Wildcard, FeatureType)
        elif Recursive == "false":
            if Wildcard:
                WorkspaceList = arcpy.ListFeatureClasses(Wildcard, FeatureType)
            else:
                WorkspaceList = arcpy.ListFeatureClasses("", FeatureType, "")
        
        arcpy.AddMessage(WorkspaceList)
        s_list = [] #list of inputs for arcpy.Merge
        for s in WorkspaceList:
            s_list.append(s)
            sDesc = arcpy.Describe(s)
            sName = sDesc.name
            if sName.endswith('.shp'):
                sName = sName[:-4]
            arcpy.AddMessage(sName)
            arcpy.AddField_management(s, FieldName, "TEXT", "", "", "124", "", "NULLABLE", "NON_REQUIRED", "")
            expression = """'{}'""".format(sName)
            arcpy.CalculateField_management(s, FieldName, expression, "PYTHON_9.3")
            #Can't use append due to fields with spaces. Try merge instead.
            #arcpy.Append_management(s, outputFC, "NO_TEST", "", "")
            
        arcpy.Merge_management(s_list, outputFC, "")
        return

class TopologyCheck(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Topology Check"
        self.description = ""
        self.canRunInBackground = False

    def getParameterInfo(self):
        """Define parameter definitions"""
        param0 = arcpy.Parameter(
            displayName="Input Feature Class",
            name="in_fc",
            datatype="GPFeatureLayer",
            parameterType="Required",
            direction="Input")
        param1 = arcpy.Parameter(
            displayName="Cluster Tolerance",
            name="cluster_tolerance",
            datatype="GPString",
            parameterType="Optional",
            direction="Input")
        parameters = [param0, param1]
        return parameters

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def execute(self, parameters, messages):
        inputFC = parameters[0].valueAsText
        clusterToleranceIn = parameters[1].valueAsText
        clusterTolerance = ""
        if clusterToleranceIn:
            clusterTolerance = float(clusterToleranceIn)
        inputFCdesc = arcpy.Describe(inputFC)
        inFCshape = inputFCdesc.shapeType
        inputFCpath = inputFCdesc.path
        inputFCname = inputFCdesc.name
        inputFCfullpath = inputFCdesc.catalogPath
        outputTopologyName = inputFCname + "_Topology"
        outputTopology = inputFCpath + "\\" + outputTopologyName
        arcpy.AddMessage(outputTopology)
#Create topology and add FC
        arcpy.CreateTopology_management(inputFCpath, outputTopologyName, clusterTolerance)
        arcpy.AddFeatureClassToTopology_management(outputTopology, inputFC, 1, 1)
#Add rules based on geometry type
        if inFCshape == "Polygon":
            arcpy.AddRuleToTopology_management(outputTopology, "Must Not Have Gaps (Area)", inputFC)
            arcpy.AddRuleToTopology_management(outputTopology, "Must Not Overlap (Area)", inputFC)
        elif inFCshape == "Polyline":
            arcpy.AddRuleToTopology_management(outputTopology, "Must Not Have Dangles (Line)", inputFC)
            arcpy.AddRuleToTopology_management(outputTopology, "Must Not Have Pseudo-Nodes (Line)", inputFC)
            arcpy.AddRuleToTopology_management(outputTopology, "Must Not Self-Overlap (Line)", inputFC)
            arcpy.AddRuleToTopology_management(outputTopology, "Must Not Intersect (Line)", inputFC)
        else:
            arcpy.ErrorMessage("This tool only support polygon and polyline geometry types")
            return
#Validate topology and export error report
        arcpy.ValidateTopology_management(outputTopology)
        arcpy.ExportTopologyErrors_management(outputTopology, inputFCpath, inputFCname)
#Collate results and delete intermediate tables
        
        return

class ZonalStats(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Zonal Statistics Tool"
        self.description = ""
        self.canRunInBackground = False
    def getParameterInfo(self):
        param0 = arcpy.Parameter(
            displayName="Output Location",
            name="in_workspace",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        param1 = arcpy.Parameter(
            displayName="Name",
            name="in_name",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        param2 = arcpy.Parameter(
            displayName="Raster Directory",
            name="in_slopedir",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        param3 = arcpy.Parameter(
            displayName="Masking Feature Class",
            name="in_ellipse",
            datatype="GPFeatureLayer",
            parameterType="Optional",
            direction="Input")
        param4 = arcpy.Parameter(
            displayName="Mask Field",
            name="in_ellipseField",
            datatype="Field",
            parameterType="Optional",
            direction="Input")
        param5 = arcpy.Parameter(
            displayName="Geologic / Landcover Map",
            name="in_features",
            datatype="GPFeatureLayer",
            parameterType="Required",
            direction="Input")
        param6 = arcpy.Parameter(
            displayName="Zonal Statistics",
            name="in_statsField",
            datatype="GPValueTable",
            parameterType="Required",
            direction="Input",
            multiValue="True")
        param4.parameterDependencies = [param3.name]
        param6.parameterDependencies = [param5.name]
        param6.columns = [["Field", "Field"], ["String", "Statistic Type"]]
        param6.filters[1].type = "ValueList"
        param6.filters[1].list = ["MEAN", "MAJORITY", "MAXIMUM", "MEDIAN", "MINIMUM", "MINORITY", "RANGE", "STD", "SUM", "VARIETY", "ALL"]
        parameters = [param0, param1, param2, param3, param4, param5, param6]

        return parameters

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def execute(self, parameters, messages):
        def dbf2csv(dbfpath, csvpath):
            rows = arcpy.SearchCursor(dbfpath)
            csvFile = csv.writer(open(csvpath, 'wb')) #output csv
            fieldnames = [f.name for f in arcpy.ListFields(dbfpath)]
            allRows = []
            for row in rows:
                rowlist = []
                for field in fieldnames:
                    rowlist.append(row.getValue(field))
                allRows.append(rowlist)
            csvFile.writerow(fieldnames)
            for row in allRows:
                csvFile.writerow(row)
            row = None
            rows = None
            return
        
        def unique_values(table, field):
            data = arcpy.da.TableToNumPyArray(table, [field])
            return np.unique(data[field])
        
        def histogram_reclass(incList, incsvpath, outcsvpath):
            csvFileName, csvExtension = os.path.splitext(incsvpath)
            csvTableView = arcpy.MakeTableView_management(incsvpath, csvFileName + "TableView", "", "",  "")
            rows = arcpy.SearchCursor(csvTableView)
            csvFile = csv.writer(open(outcsvpath, 'wb')) #output csv
            fieldnames = [f.name for f in arcpy.ListFields(csvTableView)]
            allRows = []
            for row in rows:
                rowlist = []
                for field in fieldnames:
                    m = 0
                    inc = incList[m] / 255
                    m = m + 1
                    rowlist.append(float(row.getValue(field)) * inc)
                allRows.append(rowlist)
            csvFile.writerow(fieldnames)
            for row in allRows:
                csvFile.writerow(row)
            row = None
            rows = None
            return
            
        """The source code of the tool."""  

        arcpy.CheckOutExtension("Spatial")
        workspace = parameters[0].valueAsText
        name = parameters[1].valueAsText
        SlopeDir = parameters[2].valueAsText
        Ellipse = parameters[3].valueAsText
        if Ellipse:
            EllipseDesc = arcpy.Describe(Ellipse)
            EllipseFC = EllipseDesc.catalogPath
        EllipseField = parameters[4].valueAsText
        GeoUnits = parameters[5].valueAsText
        GeoUnitsDesc = arcpy.Describe(GeoUnits)
        GeoUnitsFC = GeoUnitsDesc.catalogPath
        StatList = parameters[6].values
        StatField = parameters[6].values[0][0]
        arcpy.env.workspace = workspace
        
#Read in all rasters in slope directory, get sr and pixel type from first
        TiffList = []  
        walk = arcpy.da.Walk(SlopeDir, topdown=True, datatype="RasterDataset")  
        for dirpath, dirnames, filenames in walk:  
            for filename in filenames:  
                TiffList.append(os.path.join(dirpath, filename))
        inputRaster = TiffList[0]
        inputRasterDesc = arcpy.Describe(inputRaster)
        sr = inputRasterDesc.spatialReference
        pt = inputRasterDesc.pixelType
        if pt == "U32":
            pt = "32_BIT_UNSIGNED"
        elif pt == "S32":
            pt = "32_BIT_SIGNED"
        elif pt == "F32":
            pt = "32_BIT_FLOAT"
        elif pt == "F64":
            pt = "64_BIT"
        elif pt == "U16":
            pt = "16_BIT_UNSIGNED"
        elif pt == "S16":
            pt = "16_BIT_SIGNED"
        elif pt == "U8":
            pt = "8_BIT_UNSIGNED"
        elif pt == "S8":
            pt = "8_BIT_SIGNED"
        else:
            arcpy.AddMessage("Pixel type is not supported")
            return
        cellsize = inputRasterDesc.meanCellHeight
        extension = inputRasterDesc.format
        if extension == "TIFF":
            extension = "TIF"
#Create directory and File GDB
        newPath = workspace + "\\" + name + "\\"
#Create new directory for all outputs to be saved in
        if not os.path.exists(newPath):
            os.makedirs(newPath)
        gdb = arcpy.CreateFileGDB_management(newPath, name)
        gdbDesc = arcpy.Describe(gdb)
        gdbPath = gdbDesc.catalogPath
#Rasterize geologic units
        if GeoUnitsDesc.dataType == "FeatureClass":
            RasterizeOut = gdbPath + "\\" + "TerrainRaster"
            arcpy.PolygonToRaster_conversion(GeoUnitsFC, StatField, RasterizeOut, "MAXIMUM_COMBINED_AREA", "", cellsize)
            GeoUnitsFC = RasterizeOut
#Create and edit mosaic dataset
        mosaicName = name + "_mosaic"
        mosaic = arcpy.CreateMosaicDataset_management(gdbPath, mosaicName, sr, 1, pt, "#", "#")
        mosaicDesc = arcpy.Describe(mosaic)
        mosaicPath = mosaicDesc.catalogPath
        arcpy.AddRastersToMosaicDataset_management(mosaicPath, "Raster dataset", SlopeDir, "UPDATE_CELL_SIZES", "UPDATE_BOUNDARY", "NO_OVERVIEWS", "#", "#", "#", "#", "*.tif", "NO_SUBFOLDERS", "OVERWRITE_DUPLICATES", "NO_PYRAMIDS", "CALCULATE_STATISTICS", "NO_THUMBNAILS", "#", "NO_FORCE_SPATIAL_REFERENCE", "ESTIMATE_STATISTICS", "#")

##        try:
        if not Ellipse:
#Create table and add fields to match schema of zonal stat tables
            StatListValues = []
            for s in StatList:
                StatListValues.append(s[1])
#Perform zonal stats on each geologic unit
            statPath = newPath + "\\" + "Stat_Tables" + "\\"
            os.makedirs(statPath)
            for z in StatListValues:
                zTable = statPath + "ZonalStat_" + z + ".dbf"
                arcpy.sa.ZonalStatisticsAsTable(GeoUnitsFC, StatField, mosaicPath, zTable, "DATA", z)
                arcpy.DeleteField_management(zTable, ["ZONE_CODE", "COUNT", "AREA"])
            arcpy.env.workspace = statPath
            tableList = arcpy.ListTables("*", "ALL")
            gTable = tableList[0]
            gTable = arcpy.Rename_management(gTable, statPath + name + "_ZonalStatistics.dbf", "")
            tableLen = len(tableList)
            if tableLen > 1:
                for j in tableList[1:tableLen]:
                    arcpy.JoinField_management(gTable, StatField, j, StatField, s[1])
                    arcpy.Delete_management(j)
            gTableCSV = statPath + name + "_ZonalStatisticsJoined.csv"
            dbf2csv(gTable, gTableCSV)
            arcpy.Delete_management(gTable)
            statMaxTable = statPath + name + "_max.dbf"
            arcpy.sa.ZonalStatisticsAsTable(GeoUnitsFC, StatField, mosaicPath, statMaxTable, "DATA", "MAXIMUM")
#Create zonal histogram
            cTable = statPath + name + "_CumulativeStats.dbf"
            arcpy.sa.ZonalHistogram(GeoUnitsFC, StatField, mosaicPath, cTable)
            cTableFieldListRaw = arcpy.ListFields(cTable)
            cTableFieldList = []
            for e in cTableFieldListRaw:
                cTableFieldList.append(e.name)
            statMaxList = [row[0] for row in arcpy.da.SearchCursor(statMaxTable, "MAX")]
            statMax = max(statMaxList)
            OIDinc = statMax / 255
            cTableFieldList.insert(0, OIDinc)
            cTableFileName, cTableExtension = os.path.splitext(cTable)
            cTableReclass = cTableFileName + "_Histogram.csv"
            cTableCSV = statPath + name + "_CumulativeStats.csv"
            dbf2csv(cTable, cTableCSV)
            histogram_reclass(statMaxList, cTable, cTableReclass)
            arcpy.Delete_management(cTableCSV)
            arcpy.Delete_management(statMaxTable)
            arcpy.Delete_management(cTable)
            arcpy.env.workspace = workspace
            
        else:
            ellipseCursor = arcpy.SearchCursor(EllipseFC)
            for row in ellipseCursor:
                ellipseName = row.getValue(EllipseField)
                ellipsePath = newPath + ellipseName + "\\"
                os.makedirs(ellipsePath)
                geom = row.shape
                mosaicClip = ellipsePath + ellipseName + "_clip.tif"
#Clip mosaic to Ellipse
                arcpy.Clip_management(mosaicPath, "#", mosaicClip, geom, "0", "ClippingGeometry")
#Create table and add fields to match schema of zonal stat tables
                StatListValues = []
                for s in StatList:
                    StatListValues.append(s[1])
#Perform zonal stats on each geologic unit
                statPath = ellipsePath + "\\" + "Stat_Tables" + "\\"
                os.makedirs(statPath)
                for z in StatListValues:
                    zTable = statPath + "ZonalStat_" + z + ".dbf"
                    arcpy.sa.ZonalStatisticsAsTable(GeoUnitsFC, StatField, mosaicPath, zTable, "DATA", z)
                    arcpy.DeleteField_management(zTable, ["ZONE_CODE", "COUNT", "AREA"])
                arcpy.env.workspace = statPath
                tableList = arcpy.ListTables("*", "ALL")
                gTable = tableList[0]
                gTable = arcpy.Rename_management(gTable, statPath + name + "_ZonalStatistics.dbf", "")
                tableLen = len(tableList)
                if tableLen > 1:
                    for j in tableList[1:tableLen]:
                        arcpy.JoinField_management(gTable, StatField, j, StatField, s[1])
                        arcpy.Delete_management(j)
                gTableCSV = statPath + name + "_ZonalStatisticsJoined.csv"
                dbf2csv(gTable, gTableCSV)
                arcpy.Delete_management(gTable)
                statMaxTable = statPath + name + "_max.dbf"
                arcpy.sa.ZonalStatisticsAsTable(GeoUnitsFC, StatField, mosaicPath, statMaxTable, "DATA", "MAXIMUM")
#Create zonal histogram
                cTable = statPath + name + "_CumulativeStats.dbf"
                arcpy.sa.ZonalHistogram(GeoUnitsFC, StatField, mosaicPath, cTable)
                cTableFieldListRaw = arcpy.ListFields(cTable)
                cTableFieldList = []
                for e in cTableFieldListRaw:
                    cTableFieldList.append(e.name)
                arcpy.AddMessage(cTableFieldList)
                m = 0
                statMaxList = [row[0] for row in arcpy.da.SearchCursor(statMaxTable, "MAX")]
                statMax = max(statMaxList)
                OIDinc = statMax / 255
                cTableFieldList.insert(0, OIDinc)
                cTableFileName, cTableExtension = os.path.splitext(cTable)
                cTableReclass = cTableFileName + "_Histogram.csv"
                cTableCSV = statPath + name + "_CumulativeStats.csv"
                dbf2csv(cTable, cTableCSV)
                histogram_reclass(statMaxList, cTable, cTableReclass)
                arcpy.Delete_management(cTableCSV)
                arcpy.Delete_management(statMaxTable)
                arcpy.Delete_management(cTable)
                arcpy.env.workspace = workspace
            
class HistogramPlot(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Create Histogram Plot"
        self.description = ""
        self.canRunInBackground = False

    def getParameterInfo(self):
        param0 = arcpy.Parameter(
            displayName="Input Histogram Table",
            name="in_file",
            datatype="DETable",
            parameterType="Required",
            direction="Input")
        param1 = arcpy.Parameter(
            displayName="Title",
            name="in_title",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        param2 = arcpy.Parameter(
            displayName="X Axis Label",
            name="x_label",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        param3 = arcpy.Parameter(
            displayName="Plot Type",
            name="in_plotType",
            datatype="GPString",
            parameterType="Required",
            direction="Input",
            multiValue="True")
        parameters = [param0, param1, param2, param3]
        param3.filter.list = ["Cumulative Distribution", "Merged Histogram", "Separate Histograms"]
        return parameters

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
#input parameters
        infile = parameters[0].valueAsText
        plotTitle = parameters[1].valueAsText
        xLabel = parameters[2].valueAsText
        plotTypeList = parameters[3].values
        infileDesc = arcpy.Describe(infile)
        infileLoc = infileDesc.path
#create dataframe
        for p in plotTypeList:
            df = pd.DataFrame.from_csv(infile)
            plotTitleStr = str(plotTitle + " " + p)
            plotType = "Histogram"
            if p == "Cumulative Distribution":
                df = df.cumsum()
                df = df/df.max().astype(np.float64)
                plotType = "Cumulative_Distribution"
#plot and save out PNG
            if p == "Cumulative Distribution":
                plt.figure(); df.plot();
                ax = df.plot(title=plotTitleStr)
                ax.set_xlabel(xLabel)
                ax.yaxis.set_ticks(np.arange(0, 1, 0.1))
                outfile = infileLoc + "\\" + plotTitle + "_" + plotType + ".png"
                plt.savefig(outfile)
                print "Graph exported to %s" % (outfile)
            elif p == "Merged Histogram":
                plt.figure(); df.plot();
                ax = df.plot(title=plotTitleStr)
                ax.set_xlabel(xLabel)
                outfile = infileLoc + "\\" + plotTitle + "_" + plotType + ".png"
                plt.savefig(outfile)
            elif p == "Separate Histograms":
                dfList = list(df)
                for h in dfList:
                    plotTitleStr = plotTitle + h
                    df1 = df[h]
                    plt.figure(); df1.plot();
                    ax = df1.plot(title=plotTitleStr)
                    ax.set_xlabel(xLabel)
                    outfile = infileLoc + "\\" + plotTitleStr + "_" + "_" + plotType + ".png"
                    plt.savefig(outfile)
        return

